import re
from odoo.upgrade import util


def replace_in_all_values(cr, table, column, old, new, extra_filter=None):
    re_old = old if isinstance(old, util.PGRegexp) else r"\y{}\y".format(re.escape(old))

    if extra_filter is None:
        extra_filter = "true"

    query = cr.mogrify(
        """
        UPDATE "{table}" t
        SET "{column}" = regexp_replace(t."{column}", %(old)s, %(new)s, 'g')
        WHERE t."{column}" ~ %(old)s
          AND {extra_filter}
        """.format(
            **locals()
        ),
        {'old': re_old, 'new': new},
    ).decode()
    if "{parallel_filter}" in query:
        util.explode_execute(cr, query, table=table, alias="t")
    else:
        cr.execute(query)


def replace_in_template(cr, template_id, fields, vals):
    extra_filter = cr.mogrify("id = %s", (template_id,)).decode()
    for field in fields:
        jsonb_column = util.column_type(cr, "mail_template", field) == "jsonb"
        if jsonb_column:
            replace_values = util.replace_in_all_jsonb_values
        else:
            replace_values = replace_in_all_values
        for src, dst in vals.items():
            replace_values(cr, "mail_template", field, src, dst, extra_filter=extra_filter)

def migrate(cr, version):

    # Update Salary Offer mail & view templates
    util.update_record_from_xml(cr, "hr_contract_salary.salary_package")
    salary_offer_model_id = util.ref(cr, "hr_contract_salary.model_hr_contract_salary_offer")
    saloffer_tmpl_id = util.ref(cr, "hr_contract_salary.mail_template_send_offer")
    saloffer_applicant_tmpl_id = util.ref(cr, "hr_contract_salary.mail_template_send_offer_applicant")
    cr.execute("""
          UPDATE mail_template mailtmpl
             SET model_id = m.id, model = m.model
            FROM ir_model m
           WHERE m.id = %s AND mailtmpl.id IN %s
        """, (
        salary_offer_model_id, (saloffer_tmpl_id, saloffer_applicant_tmpl_id),
    ))
    common_replaces = {
        # updated salary simultion link
        util.PGRegexp(r"ctx.get\('salary_package_url'\)"): \
            r"'/salary_package/simulation/offer/%s?token=%s' % (ctx.get('offer_id'), ctx.get('access_token'))",
    }
    replace_in_template(cr, saloffer_tmpl_id, ["subject", "lang", "body_html"], dict(common_replaces, **{
        "object.name": "object.employee_contract_id.name",
    }))
    replace_in_template(cr, saloffer_applicant_tmpl_id, ["subject", "lang", "body_html"], dict(common_replaces, **{
        "object.name": "object.applicant_id.name",
        "object.partner_name": "object.applicant_id.partner_name",
        "object.partner_id.": "object.applicant_id.partner_id.",
    }))
    # update salary grid website action link
    # (note: we removed the `customer_relation` arg as it's the default, see odoo/enterprise@aa2e705e1d1)
    cr.execute(r"""
    UPDATE ir_act_server
        SET code = regexp_replace(code,
                                  '\ycontroller =.*',
                                  'controller = "/salary_package/simulation/contract/%s?final_yearly_costs=%s&job_id=%s&new_car=1"',
                                  'n')
        WHERE id = 3150;
    """)

    # https://github.com/odoo/odoo/pull/128281: task `is_private` removal
    cr.execute("UPDATE ir_rule SET domain_force = %(domain)s WHERE id = %(id)s", {
        "id": 1244, "domain": "['|', ('user_ids', 'in', [user.id]), ('project_id', '!=', False)]",
    })

    # https://github.com/odoo/enterprise/pull/45465: appraisals' managers now
    # need to access the appraisals survey (access_token) to see the feedback
    # answers, enable the record rule that will allow them to do that
    cr.execute("UPDATE ir_rule SET active = true WHERE id = 1275")

    # Delete record rule allowing user to view private contact as there no more
    # such contact type, and it make /web crash on init_messaging for some
    # user r- esulting in a blank page (those users only have that rule for
    # res.partner, preventing them to read their own user)
    cr.execute("DELETE FROM ir_rule WHERE id = 1230")

    # https://github.com/odoo/odoo/pull/123293: update recruitment stages
    # - hr_recruitment.stage_job0 reassigned to existing "New" stage
    # - hr_recruitment.stage_job1 reassigned to existing "Qualified" stage
    #   (making openerp_enterprise.hr_recruitment_stage_qualified xmlid useless)
    # - the new stage "New" is removed
    new_stage0_id = util.ref(cr, 'hr_recruitment.stage_job0')
    util.ensure_xmlid_match_record(cr, "hr_recruitment.stage_job0", "hr.recruitment.stage", {"id": 20})
    util.ensure_xmlid_match_record(cr, "hr_recruitment.stage_job1", "hr.recruitment.stage", {"id": 14})
    cr.execute("DELETE FROM ir_model_data WHERE module = %s AND name = %s",
               ("openerp_enterprise", "hr_recruitment_stage_qualified"))  # now stage_job1
    cr.execute("DELETE FROM hr_recruitment_stage WHERE id = %s", (new_stage0_id,))

    # deprecated views
    util.remove_view(cr, view_id=10416)  # employee form private addr. custom.
    util.remove_view(cr, "openerp_enterprise.openerp_enterprise_thanks_upgrade")
    util.remove_view(cr, "openerp_enterprise.contract_linker")
